unit UsbTypes;

interface

Uses Windows, Types, SysUtils;


Const
 USB_REQUEST_GET_STATUS        = $00;
 USB_REQUEST_CLEAR_FEATURE     = $01;
 USB_REQUEST_SET_FEATURE       = $03;
 USB_REQUEST_SET_ADDRESS       = $05;
 USB_REQUEST_GET_DESCRIPTOR    = $06;
 USB_REQUEST_SET_DESCRIPTOR    = $07;
 USB_REQUEST_GET_CONFIGURATION = $08;
 USB_REQUEST_SET_CONFIGURATION = $09;
 USB_REQUEST_GET_INTERFACE     = $0A;
 USB_REQUEST_SET_INTERFACE     = $0B;
 USB_REQUEST_SYNC_FRAME        = $0C;

 USB_GETSTATUS_SELF_POWERED          = $01;
 USB_GETSTATUS_REMOTE_WAKEUP_ENABLED = $02;

 USB_DEVICE_DESCRIPTOR_TYPE          = $01;
 USB_CONFIGURATION_DESCRIPTOR_TYPE   = $02;
 USB_STRING_DESCRIPTOR_TYPE          = $03;
 USB_INTERFACE_DESCRIPTOR_TYPE       = $04;
 USB_ENDPOINT_DESCRIPTOR_TYPE        = $05;
 USB_RESERVED_DESCRIPTOR_TYPE        = $06;
 USB_CONFIG_POWER_DESCRIPTOR_TYPE    = $07;
 USB_INTERFACE_POWER_DESCRIPTOR_TYPE = $08;
 USB_HID_DESCRIPTOR_TYPE             = $21;
 USB_REPORT_DESCRIPTOR_TYPE          = $22;

Type
 TDeviceName = packed record
  Length : ULONG;
  Name   : Array [1..256] of WCHAR;
 End;

 TUsbHubNode = (UsbHub, UsbMIParent);

 TUsbConnectionStatus = (
   NoDeviceConnected = 0,   //   
   DeviceConnected,         //  
   DeviceFailedEnumeration, //  
   DeviceGeneralFailure,    //  
   DeviceCausedOvercurrent, //   
   DeviceNotEnoughPower,    //   
   DeviceNotEnoughBandwidth //     
 );

 THubDescriptor = packed record
  bDescriptorLength   : Byte;  //   
  bDescriptorType     : Byte;  //  
  bNumberOfPorts      : Byte;  //   
  wHubCharacteristics : Word;  // 
  bPowerOnToPowerGood : Byte;  // port power on till power good in 2ms
  bHubControlCurrent  : Byte;  //   ()
  bRemoveAndPowerMask : Array [1..64] of Byte;
 End;

 TNodeInformation = packed record
  NodeType        : DWord; { = TUsbHubNode}
  HubDescriptor   : THubDescriptor;
  HubIsBusPowered : BOOL;
 End;

 TDeviceDescriptor = packed record
  bLength            : Byte;
  bDescriptorType    : Byte;
  bcdUSB             : Word;
  bDeviceClass       : Byte;
  bDeviceSubClass    : Byte;
  bDeviceProtocol    : Byte;
  bMaxPacketSize0    : Byte;
  idVendor           : Word;
  idProduct          : Word;
  bcdDevice          : Word;
  iManufacturer      : Byte;
  iProduct           : Byte;
  iSerialNumber      : Byte;
  bNumConfigurations : Byte;
 End;

 TUsbEndPointDescriptor = packed record
  bLength          : UCHAR;
  bDescriptorType  : UCHAR;
  bEndpointAddress : UCHAR;
  bmAttributes     : UCHAR;
  wMaxPacketSize   : SHORT;
  bInterval        : UCHAR;
 End;

 TUsbPipeInfo = packed record
  EndpointDescriptor : TUsbEndPointDescriptor;
  ScheduleOffset     : DWord;
 End;

 TNodeConnectionInformation = packed record
  ConnectionIndex           : DWord;
  DeviceDescriptor          : TDeviceDescriptor;
  CurrentConfigurationValue : Byte;
  LowSpeed                  : Boolean;
  DeviceIsHub               : Boolean;
  DeviceAddress             : Word;
  NumberOfOpenPipes         : Cardinal;
  ConnectionStatus          : Array [1.. 4] of Byte; {TUsbConnectionStatus}
  PipeList                  : Array [1..32] of TUsbPipeInfo;
 End;

 TSetupPacket = packed record
   bmRequest : UCHAR;
   bRequest  : UCHAR;
   wValue    : Array [1..2] of UCHAR; 
   wIndex    : Array [1..2] of UCHAR; 
   wLength   : Array [1..2] of UCHAR; 
 End;

 TDescriptorRequest = packed record
  ConnectionIndex : ULONG;
  SetupPacket     : TSetupPacket;
  Data            : Array [1..2048] of Byte;
 End;

 TUsbDeviceDescriptor = packed record
  bLength            : BYTE;
  bDescriptorType    : BYTE;
  bcdUSB             : WORD;
  bDeviceClass       : BYTE;
  bDeviceSubClass    : BYTE;
  bDeviceProtocol    : BYTE;
  bMaxPacketSize0    : BYTE;
  idVendor           : WORD;
  idProduct          : WORD;
  bcdDevice          : WORD;
  iManufacturer      : BYTE;
  iProduct           : BYTE;
  iSerialNumber      : BYTE;
  bNumConfigurations : BYTE;
 End;

 TUsbConfigurationDescriptor = packed record
  bLength             : BYTE;
  bDescriptorType     : BYTE;
  wTotalLength        : WORD;
  bNumInterfaces      : BYTE;
  bConfigurationValue : BYTE;
  iConfiguration      : BYTE;
  bmAttributes        : BYTE;
  MaxPower            : BYTE;
 End;

 TUsbInterfaceDescriptor = packed record
  bLength            : BYTE;
  bDescriptorType    : BYTE;
  bInterfaceNumber   : BYTE;
  bAlternateSetting  : BYTE;
  bNumEndpoints      : BYTE;
  bInterfaceClass    : BYTE;
  bInterfaceSubClass : BYTE;
  bInterfaceProtocol : BYTE;
  iInterface         : BYTE;
 End;

 TUsbStringDescriptor = packed record
  bLength         : BYTE;
  bDescriptorType : BYTE;
  bString         : Array of Byte;
 End;

function GetUSBClassName(IDClass : Byte) : String;
function GetUSBCtlCode(Code : Integer): Integer;
function BCD2Str(Value : Word) : String;

implementation

Const
 FILE_DEVICE_USB = $00000022;
 METHOD_BUFFERED = $00000000;
 FILE_ANY_ACCESS = $00000000;
 USB_IOCTL_INDEX = $00FF;

function GetUSBCtlCode(Code : Integer): Integer;
begin
  Result:=
     (FILE_DEVICE_USB shl 16) or
     (FILE_ANY_ACCESS shl 14) or
     ((USB_IOCTL_INDEX + Code) shl 2) or
     METHOD_BUFFERED;
end;

function GetUSBClassName(IDClass : Byte) : String;
begin
 Case IDClass of
   0: Result:= 'Reserved';
   1: Result:= 'Audio';
   2: Result:= 'Communications';
   3: Result:= 'Human Interface';
   4: Result:= 'Monitor';
   5: Result:= 'Physical Interface';
   6: Result:= 'Power';
   7: Result:= 'Printer';
   8: Result:= 'Storage';
   9: Result:= 'Hub';
  10: Result:= 'Vendor Specific';
  Else Result:= '';
 End;
end;

function BCD2Str(Value : Word) : String;
begin
 Result:= IntToHex(Value shr 8, 2) + '.' + IntToHex(Value and $00FF, 2);
end;

end.
